@@ -47,13 +47,10 @@ struct _CutFactoryBuilderClass
     void         (*set_option_context) (CutFactoryBuilder *builder,
                                         GOptionContext *context);
     GList       *(*build)              (CutFactoryBuilder *builder);
+    GList       *(*build_all)          (CutFactoryBuilder *builder);
     const gchar *(*get_type_name)      (CutFactoryBuilder *builder);
 };
 
-gboolean      cut_factory_builder_has_builder    (const gchar *type_name);
-GObject      *cut_factory_builder_create         (const gchar *type_name);
-GList        *cut_factory_builder_create_default (void);
-
 GType        cut_factory_builder_get_type       (void) G_GNUC_CONST;
 const gchar *cut_factory_builder_get_type_name  (CutFactoryBuilder *builder);
 const gchar *cut_factory_builder_get_module_dir (CutFactoryBuilder *builder);
@@ -61,6 +58,7 @@ void         cut_factory_builder_set_option_context
                                                 (CutFactoryBuilder *builder,
                                                  GOptionContext *context);
 GList       *cut_factory_builder_build          (CutFactoryBuilder *builder);
+GList       *cut_factory_builder_build_all      (CutFactoryBuilder *builder);
 
 G_END_DECLS
 
