@@ -38,6 +38,7 @@
 #include "cut-test-suite.h"
 #include "cut-ui.h"
 #include "cut-module-factory.h"
+#include "cut-contractor.h"
 #include "cut-value-equal.h"
 
 static gboolean initialized = FALSE;
@@ -48,6 +49,7 @@ static const gchar **test_names = NULL;
 static CutOrder test_case_order = CUT_ORDER_NONE_SPECIFIED;
 static gboolean use_multi_thread = FALSE;
 static GList *factories = NULL;
+static CutContractor *contractor = NULL;
 
 static gboolean
 print_version (const gchar *option_name, const gchar *value,
@@ -79,6 +81,24 @@ parse_test_case_order (const gchar *option_name, const gchar *value,
     return TRUE;
 }
 
+static gboolean
+print_help_all (const gchar *option_name, const gchar *value,
+                gpointer data, GError **error)
+{
+    GOptionContext *context = data;
+    gchar *help;
+
+    cut_contractor_build_all_factories(contractor);
+
+    help = g_option_context_get_help(context, FALSE, NULL);
+    g_print("%s", help);
+    g_free(help);
+    g_option_context_free(context);
+    exit(0);
+
+    return TRUE;
+}
+
 static const GOptionEntry option_entries[] =
 {
     {"version", 0, 0, G_OPTION_ARG_CALLBACK, print_version,
@@ -93,6 +113,8 @@ static const GOptionEntry option_entries[] =
      N_("Run test cases with multi-thread"), NULL},
     {"test-case-order", 0, 0, G_OPTION_ARG_CALLBACK, parse_test_case_order,
      N_("Sort test case by. Default is 'none'."), "[none|name|name-desc]"},
+    {"help-all", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_CALLBACK, print_help_all,
+     NULL, NULL},
     {NULL}
 };
 
@@ -100,6 +122,7 @@ void
 cut_init (int *argc, char ***argv)
 {
     GOptionContext *option_context;
+    GOptionGroup *main_group;
     GError *error = NULL;
 
     if (initialized)
@@ -120,13 +143,16 @@ cut_init (int *argc, char ***argv)
     cut_value_equal_init();
 
     option_context = g_option_context_new("TEST_DIRECTORY");
-    g_option_context_add_main_entries(option_context, option_entries, "cutter");
     g_option_context_set_help_enabled(option_context, FALSE);
     g_option_context_set_ignore_unknown_options(option_context, TRUE);
 
-    cut_module_factory_init();
-    cut_module_factory_load(NULL);
-    cut_module_factory_set_option_context(option_context);
+    contractor = cut_contractor_new();
+    cut_contractor_set_option_context(contractor, option_context);
+
+    main_group = g_option_group_new(NULL, NULL, NULL, option_context, NULL);
+    g_option_group_add_entries(main_group, option_entries);
+    g_option_group_set_translation_domain(main_group, "cutter");
+    g_option_context_set_main_group(option_context, main_group);
 
     if (!g_option_context_parse(option_context, argc, argv, &error)) {
         g_print("%s\n", error->message);
@@ -135,8 +161,7 @@ cut_init (int *argc, char ***argv)
         exit(1);
     }
 
-
-    factories = cut_module_factory_build_factories();
+    factories = cut_contractor_build_factories(contractor);
 
     g_option_context_set_help_enabled(option_context, TRUE);
     g_option_context_set_ignore_unknown_options(option_context, FALSE);
@@ -179,7 +204,11 @@ cut_quit (void)
         return;
 
     cut_ui_quit();
-    cut_module_factory_quit();
+
+    if (contractor) {
+        g_object_unref(contractor);
+        contractor = NULL;
+    }
 
     initialized = FALSE;
 }
