@@ -40,6 +40,7 @@ static GObject *constructor  (GType                  type,
 static void         set_option_context (CutFactoryBuilder *builder,
                                         GOptionContext    *context);
 static GList       *build              (CutFactoryBuilder *builder);
+static GList       *build_all          (CutFactoryBuilder *builder);
 static const gchar *get_type_name      (CutFactoryBuilder *builder);
 
 G_DEFINE_TYPE(CutReportFactoryBuilder, cut_report_factory_builder, CUT_TYPE_FACTORY_BUILDER)
@@ -57,6 +58,7 @@ cut_report_factory_builder_class_init (CutReportFactoryBuilderClass *klass)
 
     builder_class->set_option_context = set_option_context;
     builder_class->build              = build;
+    builder_class->build_all          = build_all;
     builder_class->get_type_name      = get_type_name;
 }
 
@@ -68,6 +70,7 @@ constructor (GType type, guint n_props, GObjectConstructParam *props)
     if (!the_builder) {
         GObjectClass *klass = G_OBJECT_CLASS(cut_report_factory_builder_parent_class);
         object = klass->constructor(type, n_props, props);
+        the_builder = CUT_REPORT_FACTORY_BUILDER(object);
     } else {
         object = g_object_ref(G_OBJECT(the_builder));
     }
@@ -87,6 +90,7 @@ cut_report_factory_builder_init (CutReportFactoryBuilder *builder)
     g_object_set(G_OBJECT(builder),
                  "module-dir", dir,
                  NULL);
+    cut_module_factory_load(dir, "report");
 }
 
 static void
@@ -118,6 +122,7 @@ build (CutFactoryBuilder *builder)
         return NULL;
 
     report_type = "xml";
+
     if (cut_module_factory_exist_module("report", report_type)) {
         CutModuleFactory *module_factory;
         GOptionContext *option_context;
@@ -132,7 +137,30 @@ build (CutFactoryBuilder *builder)
     return factories;
 }
 
-const gchar *
+static GList *
+build_all (CutFactoryBuilder *builder)
+{
+    GList *factories = NULL, *node;
+    GList *factory_names;
+
+    factory_names = cut_module_factory_get_names("report");
+
+    for (node = factory_names; node; node = g_list_next(node)) {
+        CutModuleFactory *module_factory;
+        GOptionContext *option_context;
+        module_factory = cut_module_factory_new("report", node->data, NULL);
+        g_object_get(builder,
+                     "option-context", &option_context,
+                     NULL);
+        cut_module_factory_set_option_group(module_factory,
+                                            option_context);
+        factories = g_list_prepend(factories, module_factory);
+    }
+
+    return factories;
+}
+
+static const gchar *
 get_type_name (CutFactoryBuilder *builder)
 {
     return "report";
