@@ -31,6 +31,7 @@
 #include "cut-test-result.h"
 
 #include "cut-marshalers.h"
+#include "cut-enum-types.h"
 
 #define CUT_RUNNER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CUT_TYPE_RUNNER, CutRunnerPrivate))
 
@@ -55,6 +56,7 @@ struct _CutRunnerPrivate
     gboolean canceled;
     CutTestSuite *test_suite;
     GList *listeners;
+    CutOrder test_case_order;
 };
 
 enum
@@ -66,7 +68,8 @@ enum
     PROP_N_ERRORS,
     PROP_N_PENDINGS,
     PROP_N_NOTIFICATIONS,
-    PROP_USE_MULTI_THREAD
+    PROP_USE_MULTI_THREAD,
+    PROP_TEST_CASE_ORDER
 };
 
 enum
@@ -86,6 +89,12 @@ enum
     PENDING_TEST,
     NOTIFICATION_TEST,
 
+    SUCCESS_TEST_CASE,
+    FAILURE_TEST_CASE,
+    ERROR_TEST_CASE,
+    PENDING_TEST_CASE,
+    NOTIFICATION_TEST_CASE,
+
     COMPLETE_TEST,
     COMPLETE_TEST_CASE,
     COMPLETE_TEST_SUITE,
@@ -163,6 +172,14 @@ cut_runner_class_init (CutRunnerClass *klass)
                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property(gobject_class, PROP_USE_MULTI_THREAD, spec);
 
+    spec = g_param_spec_enum("test-case-order",
+                             "Test case order",
+                             "Sort key for test case",
+                             CUT_TYPE_ORDER,
+                             CUT_ORDER_NONE_SPECIFIED,
+                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+    g_object_class_install_property(gobject_class, PROP_TEST_CASE_ORDER, spec);
+
 
     signals[READY_TEST_SUITE]
         = g_signal_new ("ready-test-suite",
@@ -208,7 +225,8 @@ cut_runner_class_init (CutRunnerClass *klass)
                         G_STRUCT_OFFSET (CutRunnerClass, start_test),
                         NULL, NULL,
                         _cut_marshal_VOID__OBJECT_OBJECT,
-                        G_TYPE_NONE, 2, CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT);
+                        G_TYPE_NONE, 2,
+                        CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT);
 
     signals[PASS_TEST]
         = g_signal_new ("pass-test",
@@ -225,8 +243,10 @@ cut_runner_class_init (CutRunnerClass *klass)
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET (CutRunnerClass, success_test),
                         NULL, NULL,
-                        g_cclosure_marshal_VOID__OBJECT,
-                        G_TYPE_NONE, 1, CUT_TYPE_TEST);
+                        _cut_marshal_VOID__OBJECT_OBJECT_OBJECT,
+                        G_TYPE_NONE, 3,
+                        CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT,
+                        CUT_TYPE_TEST_RESULT);
 
     signals[FAILURE_TEST]
         = g_signal_new ("failure-test",
@@ -281,6 +301,61 @@ cut_runner_class_init (CutRunnerClass *klass)
                         _cut_marshal_VOID__OBJECT_OBJECT,
                         G_TYPE_NONE, 2, CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT);
 
+    signals[SUCCESS_TEST_CASE]
+        = g_signal_new ("success-test-case",
+                        G_TYPE_FROM_CLASS (klass),
+                        G_SIGNAL_RUN_LAST,
+                        G_STRUCT_OFFSET (CutRunnerClass, success_test_case),
+                        NULL, NULL,
+                        _cut_marshal_VOID__OBJECT_OBJECT_OBJECT,
+                        G_TYPE_NONE, 3,
+                        CUT_TYPE_TEST_CASE, CUT_TYPE_TEST_CONTEXT,
+                        CUT_TYPE_TEST_RESULT);
+
+    signals[FAILURE_TEST_CASE]
+        = g_signal_new ("failure-test-case",
+                        G_TYPE_FROM_CLASS (klass),
+                        G_SIGNAL_RUN_LAST,
+                        G_STRUCT_OFFSET (CutRunnerClass, failure_test_case),
+                        NULL, NULL,
+                        _cut_marshal_VOID__OBJECT_OBJECT_OBJECT,
+                        G_TYPE_NONE, 3,
+                        CUT_TYPE_TEST_CASE, CUT_TYPE_TEST_CONTEXT,
+                        CUT_TYPE_TEST_RESULT);
+
+    signals[ERROR_TEST_CASE]
+        = g_signal_new ("error-test-case",
+                        G_TYPE_FROM_CLASS (klass),
+                        G_SIGNAL_RUN_LAST,
+                        G_STRUCT_OFFSET (CutRunnerClass, error_test_case),
+                        NULL, NULL,
+                        _cut_marshal_VOID__OBJECT_OBJECT_OBJECT,
+                        G_TYPE_NONE, 3,
+                        CUT_TYPE_TEST_CASE, CUT_TYPE_TEST_CONTEXT,
+                        CUT_TYPE_TEST_RESULT);
+
+    signals[PENDING_TEST_CASE]
+        = g_signal_new ("pending-test-case",
+                        G_TYPE_FROM_CLASS (klass),
+                        G_SIGNAL_RUN_LAST,
+                        G_STRUCT_OFFSET (CutRunnerClass, pending_test_case),
+                        NULL, NULL,
+                        _cut_marshal_VOID__OBJECT_OBJECT_OBJECT,
+                        G_TYPE_NONE, 3,
+                        CUT_TYPE_TEST_CASE, CUT_TYPE_TEST_CONTEXT,
+                        CUT_TYPE_TEST_RESULT);
+
+    signals[NOTIFICATION_TEST_CASE]
+        = g_signal_new ("notification-test-case",
+                        G_TYPE_FROM_CLASS (klass),
+                        G_SIGNAL_RUN_LAST,
+                        G_STRUCT_OFFSET (CutRunnerClass, notification_test_case),
+                        NULL, NULL,
+                        _cut_marshal_VOID__OBJECT_OBJECT_OBJECT,
+                        G_TYPE_NONE, 3,
+                        CUT_TYPE_TEST_CASE, CUT_TYPE_TEST_CONTEXT,
+                        CUT_TYPE_TEST_RESULT);
+
     signals[COMPLETE_TEST_CASE]
         = g_signal_new ("complete-test-case",
                         G_TYPE_FROM_CLASS (klass),
@@ -414,6 +489,9 @@ set_property (GObject      *object,
       case PROP_USE_MULTI_THREAD:
         priv->use_multi_thread = g_value_get_boolean(value);
         break;
+      case PROP_TEST_CASE_ORDER:
+        priv->test_case_order = g_value_get_enum(value);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -450,6 +528,9 @@ get_property (GObject    *object,
       case PROP_USE_MULTI_THREAD:
         g_value_set_boolean(value, priv->use_multi_thread);
         break;
+      case PROP_TEST_CASE_ORDER:
+        g_value_set_enum(value, priv->test_case_order);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -580,7 +661,8 @@ cb_pass_assertion (CutTest *test, CutTestContext *test_context, gpointer data)
 }
 
 static void
-cb_success (CutTest *test, CutTestResult *result, gpointer data)
+cb_success (CutTest *test, CutTestContext *test_context, CutTestResult *result,
+            gpointer data)
 {
     CutRunner *runner = data;
     CutRunnerPrivate *priv;
@@ -590,7 +672,7 @@ cb_success (CutTest *test, CutTestResult *result, gpointer data)
     priv->results = g_list_prepend(priv->results, g_object_ref(result));
     g_mutex_unlock(priv->mutex);
 
-    g_signal_emit(runner, signals[SUCCESS_TEST], 0, test);
+    g_signal_emit(runner, signals[SUCCESS_TEST], 0, test, test_context, result);
 }
 
 static void
@@ -721,6 +803,56 @@ cb_complete_test(CutTestCase *test_case, CutTest *test,
     g_signal_emit(runner, signals[COMPLETE_TEST], 0, test, test_context);
 }
 
+static void
+cb_success_test_case (CutTestCase *test_case, CutTestContext *test_context,
+                      CutTestResult *result, gpointer data)
+{
+    CutRunner *runner = data;
+
+    g_signal_emit(runner, signals[SUCCESS_TEST_CASE], 0,
+                  test_case, test_context, result);
+}
+
+static void
+cb_failure_test_case (CutTestCase *test_case, CutTestContext *test_context,
+                      CutTestResult *result, gpointer data)
+{
+    CutRunner *runner = data;
+
+    g_signal_emit(runner, signals[FAILURE_TEST_CASE], 0,
+                  test_case, test_context, result);
+}
+
+static void
+cb_error_test_case (CutTestCase *test_case, CutTestContext *test_context,
+                    CutTestResult *result, gpointer data)
+{
+    CutRunner *runner = data;
+
+    g_signal_emit(runner, signals[ERROR_TEST_CASE], 0,
+                  test_case, test_context, result);
+}
+
+static void
+cb_pending_test_case (CutTestCase *test_case, CutTestContext *test_context,
+                      CutTestResult *result, gpointer data)
+{
+    CutRunner *runner = data;
+
+    g_signal_emit(runner, signals[PENDING_TEST_CASE], 0,
+                  test_case, test_context, result);
+}
+
+static void
+cb_notification_test_case (CutTestCase *test_case, CutTestContext *test_context,
+                           CutTestResult *result, gpointer data)
+{
+    CutRunner *runner = data;
+
+    g_signal_emit(runner, signals[NOTIFICATION_TEST_CASE], 0,
+                  test_case, test_context, result);
+}
+
 static void
 cb_start_test_case(CutTestCase *test_case, gpointer data)
 {
@@ -749,6 +881,12 @@ cb_complete_test_case(CutTestCase *test_case, gpointer data)
 
     DISCONNECT(start_test);
     DISCONNECT(complete_test);
+#undef DISCONNECT
+
+#define DISCONNECT(name)                                                \
+    g_signal_handlers_disconnect_by_func(test_case,                     \
+                                         G_CALLBACK(cb_ ## name ## _test_case), \
+                                         data)
 
     DISCONNECT(success);
     DISCONNECT(failure);
@@ -756,10 +894,9 @@ cb_complete_test_case(CutTestCase *test_case, gpointer data)
     DISCONNECT(pending);
     DISCONNECT(notification);
 
-    DISCONNECT(ready_test_case);
-    DISCONNECT(start_test_case);
-    DISCONNECT(complete_test_case);
-
+    DISCONNECT(ready);
+    DISCONNECT(start);
+    DISCONNECT(complete);
 #undef DISCONNECT
 
     g_signal_emit(runner, signals[COMPLETE_TEST_CASE], 0, test_case);
@@ -773,6 +910,11 @@ cut_runner_start_test_case (CutRunner *runner, CutTestCase *test_case)
 
     CONNECT(start_test);
     CONNECT(complete_test);
+#undef CONNECT
+
+#define CONNECT(name)                                                   \
+    g_signal_connect(test_case, #name,                                  \
+                     G_CALLBACK(cb_ ## name ## _test_case), runner)
 
     CONNECT(success);
     CONNECT(failure);
@@ -780,16 +922,9 @@ cut_runner_start_test_case (CutRunner *runner, CutTestCase *test_case)
     CONNECT(pending);
     CONNECT(notification);
 
-#undef CONNECT
-
-#define CONNECT(name)                                                   \
-    g_signal_connect(test_case, #name,                                  \
-                     G_CALLBACK(cb_ ## name ## _test_case), runner)
-
     CONNECT(ready);
     CONNECT(start);
     CONNECT(complete);
-
 #undef CONNECT
 }
 
@@ -1047,6 +1182,71 @@ cut_runner_build_source_filename (CutRunner *runner, const gchar *filename)
     return source_filename;
 }
 
+void
+cut_runner_set_test_case_order (CutRunner *runner, CutOrder order)
+{
+    CutRunnerPrivate *priv;
+
+    priv = CUT_RUNNER_GET_PRIVATE(runner);
+    g_mutex_lock(priv->mutex);
+    priv->test_case_order = order;
+    g_mutex_unlock(priv->mutex);
+}
+
+CutOrder
+cut_runner_get_test_case_order (CutRunner *runner)
+{
+    return CUT_RUNNER_GET_PRIVATE(runner)->test_case_order;
+}
+
+static gint
+compare_test_cases_by_name (gconstpointer a, gconstpointer b, gpointer user_data)
+{
+    CutTestCase *test_case1, *test_case2;
+    const gchar *test_case_name1, *test_case_name2;
+    gboolean ascending;
+
+    test_case1 = CUT_TEST_CASE(a);
+    test_case2 = CUT_TEST_CASE(b);
+    test_case_name1 = cut_test_get_name(CUT_TEST(test_case1));
+    test_case_name2 = cut_test_get_name(CUT_TEST(test_case2));
+    ascending = *(gboolean *)user_data;
+
+    if (ascending)
+        return strcmp(test_case_name1, test_case_name2);
+    else
+        return strcmp(test_case_name2, test_case_name1);
+}
+
+GList *
+cut_runner_sort_test_cases (CutRunner *runner, GList *test_cases)
+{
+    CutRunnerPrivate *priv;
+    GList *sorted_test_cases = NULL;
+    gboolean ascending;
+
+    priv = CUT_RUNNER_GET_PRIVATE(runner);
+    switch (priv->test_case_order) {
+      case CUT_ORDER_NONE_SPECIFIED:
+        sorted_test_cases = test_cases;
+        break;
+      case CUT_ORDER_NAME_ASCENDING:
+        ascending = TRUE;
+        sorted_test_cases = g_list_sort_with_data(test_cases,
+                                                  compare_test_cases_by_name,
+                                                  &ascending);
+        break;
+      case CUT_ORDER_NAME_DESCENDING:
+        ascending = FALSE;
+        sorted_test_cases = g_list_sort_with_data(test_cases,
+                                                  compare_test_cases_by_name,
+                                                  &ascending);
+        break;
+    }
+
+    return sorted_test_cases;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
