@@ -131,11 +131,12 @@ cut_test_container_add_test (CutTestContainer *container, CutTest *test)
     CutTestContainerPrivate *priv = CUT_TEST_CONTAINER_GET_PRIVATE(container);
 
     if (CUT_IS_TEST(test)) {
+        g_object_ref(test);
         priv->tests = g_list_prepend(priv->tests, test);
     }
 }
 
-const GList *
+GList *
 cut_test_container_get_children (CutTestContainer *container)
 {
     return CUT_TEST_CONTAINER_GET_PRIVATE(container)->tests;
@@ -145,7 +146,7 @@ static gdouble
 real_get_elapsed (CutTest *test)
 {
     gdouble result = 0.0;
-    const GList *child;
+    GList *child;
     CutTestContainerPrivate *priv;
 
     g_return_val_if_fail(CUT_IS_TEST_CONTAINER(test), FALSE);
