@@ -39,6 +39,7 @@ static GObject *constructor  (GType                  type,
 static void         set_option_context (CutFactoryBuilder *builder,
                                         GOptionContext    *context);
 static GList       *build              (CutFactoryBuilder *builder);
+static GList       *build_all          (CutFactoryBuilder *builder);
 static const gchar *get_type_name      (CutFactoryBuilder *builder);
 
 G_DEFINE_TYPE(CutUIFactoryBuilder, cut_ui_factory_builder, CUT_TYPE_FACTORY_BUILDER)
@@ -56,6 +57,7 @@ cut_ui_factory_builder_class_init (CutUIFactoryBuilderClass *klass)
 
     builder_class->set_option_context = set_option_context;
     builder_class->build              = build;
+    builder_class->build_all          = build_all;
     builder_class->get_type_name      = get_type_name;
 }
 
@@ -67,6 +69,7 @@ constructor (GType type, guint n_props, GObjectConstructParam *props)
     if (!the_builder) {
         GObjectClass *klass = G_OBJECT_CLASS(cut_ui_factory_builder_parent_class);
         object = klass->constructor(type, n_props, props);
+        the_builder = CUT_UI_FACTORY_BUILDER(object);
     } else {
         object = g_object_ref(G_OBJECT(the_builder));
     }
@@ -151,7 +154,30 @@ build (CutFactoryBuilder *builder)
     return factories;
 }
 
-const gchar *
+static GList *
+build_all (CutFactoryBuilder *builder)
+{
+    GList *factories = NULL, *node;
+    GList *factory_names;
+
+    factory_names = cut_module_factory_get_names("ui");
+
+    for (node = factory_names; node; node = g_list_next(node)) {
+        CutModuleFactory *module_factory;
+        GOptionContext *option_context;
+        module_factory = cut_module_factory_new("ui", node->data, NULL);
+        g_object_get(builder,
+                     "option-context", &option_context,
+                     NULL);
+        cut_module_factory_set_option_group(module_factory,
+                                            option_context);
+        factories = g_list_prepend(factories, module_factory);
+    }
+
+    return factories;
+}
+
+static const gchar *
 get_type_name (CutFactoryBuilder *builder)
 {
     return "ui";
