@@ -25,8 +25,7 @@
 #include <glib-object.h>
 
 #include <cutter/cut-private.h>
-#include <cutter/cut-test-result.h>
-#include <cutter/cut-verbose-level.h>
+#include <cutter/cut-listener.h>
 
 G_BEGIN_DECLS
 
@@ -42,15 +41,12 @@ typedef struct _CutUIClass    CutUIClass;
 
 struct _CutUI
 {
-    GObject object;
+    CutListener object;
 };
 
 struct _CutUIClass
 {
-    GObjectClass parent_class;
-
-    gboolean (*run)                (CutUI    *ui,
-                                    CutRunner   *runner);
+    CutListenerClass parent_class;
 };
 
 GType           cut_ui_get_type  (void) G_GNUC_CONST;
@@ -70,8 +66,6 @@ CutUI          *cut_ui_new         (const gchar *name,
                                     const gchar *first_property,
                                     ...);
 
-gboolean        cut_ui_run         (CutUI        *ui,
-                                    CutRunner    *runner);
 G_END_DECLS
 
 #endif /* __CUT_UI_H__ */
